@@ -8,11 +8,6 @@ pub const LINE_ENDING: &'static str = "\r\n";
 #[cfg(not(windows))]
 pub const LINE_ENDING: &'static str = "\n";
 
-#[cfg(windows)]
-pub const TEMP_PATH: &'static str = "%TEMP%";
-#[cfg(not(windows))]
-pub const TEMP_PATH: &'static str = "/tmp";
-
-pub const TEMP_FILE: &'static str = "rad_temp.txt";
+pub const TEMP_FILE: &'static str = "radtemp.txt";
 
 pub const LIT_CHAR: char = '*';
